# -*- coding: utf-8 -*-
#
# Copyright (C) 2006 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://projects.edgewall.com/trac/.

import copy
import re

from trac.config import ListOption, IntOption
from trac.core import Component, implements

from tracspamfilter.api import IFilterStrategy, N_


class ExternalLinksFilterStrategy(Component):
    """Spam filter strategy that reduces the karma of a submission if the
    content contains too many links to external sites.
    """
    implements(IFilterStrategy)

    karma_points = IntOption('spam-filter', 'extlinks_karma', '2',
        """By how many points too many external links in a submission impact
        the overall score.""", doc_domain='tracspamfilter')

    max_links = IntOption('spam-filter', 'max_external_links', '4',
        """The maximum number of external links allowed in a submission until
        that submission gets negative karma.""", doc_domain='tracspamfilter')

    allowed_domains = ListOption('spam-filter', 'extlinks_allowed_domains',
        'example.com, example.org',
        doc="List of domains that should be allowed in external links",
        doc_domain='tracspamfilter')

    _URL_RE = re.compile('https?://([^/]+)/?', re.IGNORECASE)

    # IFilterStrategy methods

    def is_external(self):
        return False

    def test(self, req, author, content, ip):
        num_ext = 0
        allowed = copy.copy(self.allowed_domains)
        allowed.append(req.get_header('Host'))

        for host in self._URL_RE.findall(content):
            if host not in allowed:
                self.log.debug('"%s" is not in extlink_allowed_domains', host)
                num_ext += 1
            else:
                self.log.debug('"%s" is whitelisted.', host)

        if num_ext > self.max_links:
            if self.max_links > 0:
                return -abs(self.karma_points) * num_ext / self.max_links, \
                       N_("Maximum number of external links per post exceeded")
            else:
                return -abs(self.karma_points) * num_ext, \
                       N_("External links in post found")

    def train(self, req, author, content, ip, spam=True):
        return 0



# -*- coding: utf-8 -*-
#
# Copyright (C) 2006 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://projects.edgewall.com/trac/.

from __future__ import with_statement

import unittest

from trac.test import EnvironmentStub, Mock

from tracspamfilter.filtersystem import FilterSystem
from tracspamfilter.tests.model import drop_tables


class BayesianFilterStrategyTestCase(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentStub(enable=[BayesianFilterStrategy])
        self.env.config.set('spam-filter', 'bayes_karma', '10')
        with self.env.db_transaction as db:
            FilterSystem(self.env).upgrade_environment(db)

        self.strategy = BayesianFilterStrategy(self.env)

    def tearDown(self):
        drop_tables(self.env)
        self.env.reset_db()

    def test_karma_calculation_unsure(self):
        bayes.Hammie = lambda x: Mock(score=lambda x: .5,
                                      bayes=Mock(nham=1000, nspam=1000))

        req = Mock(authname='anonymous', base_url='http://example.org/',
                   remote_addr='127.0.0.1')
        self.assertEquals(None, self.strategy.test(req, 'John Doe', 'Spam',
                                                   '127.0.0.1'))

    def test_karma_calculation_negative(self):
        bayes.Hammie = lambda x: Mock(score=lambda x: .75,
                                      bayes=Mock(nham=1000, nspam=1000))

        req = Mock(authname='anonymous', base_url='http://example.org/',
                   remote_addr='127.0.0.1')
        points, reasons, args = \
            self.strategy.test(req, 'John Doe', 'Spam', '127.0.0.1')
        self.assertEquals(-5, points)

    def test_karma_calculation_positive(self):
        bayes.Hammie = lambda x: Mock(score=lambda x: .25,
                                      bayes=Mock(nham=1000, nspam=1000))

        req = Mock(authname='anonymous', base_url='http://example.org/',
                   remote_addr='127.0.0.1')
        points, reasons, args = \
            self.strategy.test(req, 'John Doe', 'Spam', '127.0.0.1')
        self.assertEquals(5, points)

    def test_classifier_untrained(self):
        req = Mock(authname='anonymous', base_url='http://example.org/',
                   remote_addr='127.0.0.1')
        self.assertEqual(None, self.strategy.test(req, 'John Doe', 'Hammie',
                                                  '127.0.0.1'))

    def test_classifier_basics(self):
        req = Mock(authname='anonymous', base_url='http://example.org/',
                   remote_addr='127.0.0.1')
        self.env.config.set('spam-filter', 'bayes_min_training', '1')
        self.strategy.train(req, 'John Doe', 'Spam spam spammie', '127.0.0.1',
                            True)
        self.strategy.train(req, 'John Doe', 'Ham ham hammie', '127.0.0.1',
                            False)

        points, reasons, args = \
            self.strategy.test(req, 'John Doe', 'Hammie', '127.0.0.1')
        self.assertGreater(points, 0, 'Expected positive karma')
        points, reasons, args = \
            self.strategy.test(req, 'John Doe', 'Spam', '127.0.0.1')
        self.assertLess(points, 0, 'Expected negative karma')


try:
    from tracspamfilter.filters import bayes
    from tracspamfilter.filters.bayes import BayesianFilterStrategy
except ImportError:
    # Skip tests if SpamBayes isn't installed
    class BayesianFilterStrategyTestCase(object):
        pass


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BayesianFilterStrategyTestCase))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2006-2015 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://projects.edgewall.com/trac/.

import sys
from setuptools import setup, find_packages

PACKAGE = 'TracSpamFilter'
VERSION = '1.0.10'

extra = {}
try:
    from trac.util.dist import get_l10n_cmdclass
    cmdclass = get_l10n_cmdclass()
    if cmdclass:
        extra['cmdclass'] = cmdclass
        extractors = [
            ('**.py',                'trac.dist:extract_python', None),
            ('**/templates/**.html', 'genshi', None)
        ]
        extra['message_extractors'] = {
            'tracspamfilter': extractors,
        }
except ImportError:
    pass

setup(
    name=PACKAGE,
    version=VERSION,
    description='Plugin for spam filtering',
    author="Edgewall Software",
    author_email="info@edgewall.com",
    url='http://trac.edgewall.org/wiki/SpamFilter',
    download_url='http://trac.edgewall.org/wiki/SpamFilter',
    license='BSD',
    classifiers=[
        'Framework :: Trac',
        'License :: OSI Approved :: BSD License',
    ],
    keywords='trac plugin',

    packages=find_packages(exclude=['*.tests*']),
    package_data={'tracspamfilter': [
        'templates/*',
        'htdocs/*',
        'fonts/*',
        'locale/*/LC_MESSAGES/*.mo'
    ]},
    install_requires=['Trac'],
    extras_require={
        'dns': ['dnspython>=1.3.5'],
        'spambayes': ['spambayes'],
        'pillow': ['pillow'],
        'json': ['simplejson' if sys.version_info < (2, 6) else ''],
        'account': ['TracAccountManager >= 0.4'],
        'oauth': ['oauth2'],
        'httplib2': ['httplib2']
    },
    entry_points="""
        [trac.plugins]
        spamfilter = tracspamfilter.api
        spamfilter.filtersystem = tracspamfilter.filtersystem
        spamfilter.admin = tracspamfilter.admin
        spamfilter.adminusers = tracspamfilter.adminusers
        spamfilter.adminreport = tracspamfilter.adminreport
        spamfilter.adapters = tracspamfilter.adapters
        spamfilter.report = tracspamfilter.report
        spamfilter.accountadapter = tracspamfilter.accountadapter[account]
        spamfilter.registration = tracspamfilter.filters.registration[account]
        spamfilter.akismet = tracspamfilter.filters.akismet
        spamfilter.stopforumspam = tracspamfilter.filters.stopforumspam
        spamfilter.botscout = tracspamfilter.filters.botscout
        spamfilter.fspamlist = tracspamfilter.filters.fspamlist
        spamfilter.blogspam = tracspamfilter.filters.blogspam[json]
        spamfilter.mollom = tracspamfilter.filters.mollom[oauth,httplib2]
        spamfilter.bayes = tracspamfilter.filters.bayes[spambayes]
        spamfilter.extlinks = tracspamfilter.filters.extlinks
        spamfilter.httpbl = tracspamfilter.filters.httpbl[dns]
        spamfilter.ip_blacklist = tracspamfilter.filters.ip_blacklist[dns]
        spamfilter.url_blacklist = tracspamfilter.filters.url_blacklist[dns]
        spamfilter.ip_throttle = tracspamfilter.filters.ip_throttle
        spamfilter.regex = tracspamfilter.filters.regex
        spamfilter.trapfield = tracspamfilter.filters.trapfield
        spamfilter.ip_regex = tracspamfilter.filters.ip_regex
        spamfilter.session = tracspamfilter.filters.session
        spamfilter.captcha = tracspamfilter.captcha.api
        spamfilter.captcha.admin = tracspamfilter.captcha.admin
        spamfilter.captcha.image = tracspamfilter.captcha.image[pillow]
        spamfilter.captcha.expression = tracspamfilter.captcha.expression
        spamfilter.captcha.rand = tracspamfilter.captcha.rand
        spamfilter.captcha.recaptcha = tracspamfilter.captcha.recaptcha
        spamfilter.captcha.recaptcha2 = tracspamfilter.captcha.recaptcha2[json]
        spamfilter.captcha.keycaptcha = tracspamfilter.captcha.keycaptcha
        spamfilter.captcha.mollom = tracspamfilter.captcha.mollom[oauth,httplib2]
    """,
    test_suite='tracspamfilter.tests.test_suite',
    zip_safe=False,
    **extra
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://projects.edgewall.com/trac/.

import StringIO

from trac.perm import PermissionCache
from trac.test import Mock, MockPerm, locale_en
from trac.util.datefmt import utc
from trac.web.api import _RequestArgs, Request
from trac.web.session import DetachedSession

try:
    from trac.test import MockRequest
except ImportError:
    def MockRequest(env, **kwargs):
        """Request object for testing. Keyword arguments populate an
        `environ` dictionary and the callbacks.

        If `authname` is specified in a keyword arguments a `PermissionCache`
        object is created, otherwise if `authname` is not specified or is
        `None` a `MockPerm` object is used and the `authname` is set to
        'anonymous'.

        The following keyword arguments are commonly used:
        :keyword args: dictionary of request arguments
        :keyword authname: the name of the authenticated user, or 'anonymous'
        :keyword method: the HTTP request method
        :keyword path_info: the request path inside the application

        Additionally `format`, `locale`, `lc_time` and `tz` can be
        specified as keyword arguments.

        :since: 1.0.11
        """

        authname = kwargs.get('authname')
        if authname is None:
            authname = 'anonymous'
            perm = MockPerm()
        else:
            perm = PermissionCache(env, authname)

        args = _RequestArgs()
        args.update(kwargs.get('args', {}))

        environ = {
            'trac.base_url': env.abs_href(),
            'wsgi.url_scheme': 'http',
            'HTTP_ACCEPT_LANGUAGE': 'en-US',
            'PATH_INFO': kwargs.get('path_info', '/'),
            'REQUEST_METHOD': kwargs.get('method', 'GET'),
            'REMOTE_ADDR': '127.0.0.1',
            'REMOTE_USER': authname,
            'SCRIPT_NAME': '/trac.cgi',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '80',
        }

        status_sent = []
        headers_sent = {}
        response_sent = StringIO.StringIO()

        def start_response(status, headers, exc_info=None):
            status_sent.append(status)
            headers_sent.update(dict(headers))
            return response_sent.write

        req = Mock(Request, environ, start_response)
        req.status_sent = status_sent
        req.headers_sent = headers_sent
        req.response_sent = response_sent

        from trac.web.chrome import Chrome
        req.callbacks.update({
            'arg_list': None,
            'args': lambda req: args,
            'authname': lambda req: authname,
            'chrome': Chrome(env).prepare_request,
            'form_token': lambda req: kwargs.get('form_token'),
            'languages': Request._parse_languages,
            'lc_time': lambda req: kwargs.get('lc_time', locale_en),
            'locale': lambda req: kwargs.get('locale'),
            'incookie': Request._parse_cookies,
            'perm': lambda req: perm,
            'session': lambda req: DetachedSession(env, authname),
            'tz': lambda req: kwargs.get('tz', utc),
            'use_xsendfile': False,
            'xsendfile_header': None,
            '_inheaders': Request._parse_headers
        })

        return req